---
layout: default
title: Unit Driven
nav_order: 2
parent: Design Patterns
---

> "Unit-driven AI generation" is a method or concept that emphasizes the use of artificial intelligence (AI) generation
> targeted at specific units or modules to meet particular needs or goals. In this context, units can refer to various
> modules in software development, such as function blocks, class blocks, test methods, SQL snippets, code snippets, or
> even user requirements, among others.

The core idea of this approach is to divide the entire system or application into small, independent units and then use
AI technology to generate corresponding code, scripts, queries, or other necessary implementations for each unit. This
approach helps improve development efficiency, reduce repetitive work, and ensure that each unit can operate
independently and consistently throughout the system.

Specifically, unit-driven AI generation may involve using natural language processing (NLP) techniques to parse user
requirements and then generate corresponding code snippets. Alternatively, it could involve learning and inference to
generate corresponding unit implementations for specific programming languages, database query languages, etc. The
advantage of this approach is its flexibility in adapting to different requirements and environments while increasing
the speed and quality of development.

## Unit Driven Example

### AutoSQL

see in [AutoSQL](https://github.com/unit-mesh/auto-dev/tree/master/exts/database/src/main/kotlin/cc/unitmesh/database/flow)

### AutoPage

see in [AutoPage](https://github.com/unit-mesh/auto-dev/tree/master/javascript/src/main/kotlin/cc/unitmesh/ide/javascript/flow)

### AutoArkUI

see in [AutoArkUI](https://github.com/unit-mesh/auto-dev/tree/master/exts/ext-harmonyos/src/main/kotlin/cc/unitmesh/harmonyos/actions/auto)

